/**
 * Documentize an apemanfile.
 * @function apemanfileDocBud
 */

"use strict";

const _tmpl = require('./_tmpl'),
    path = require('path'),
    assert = require('assert'),
    arraysort = require('arraysort'),
    arrayreduce = require('arrayreduce'),
    arrayfilter = require('arrayfilter'),
    pkgData = require('./pkg_data'),
    protoData = require('./proto_data'),
    appsData = require('./apps_data'),
    tmplsData = require('./tmpls_data'),
    tasksData = require('./tasks_data'),
    wtchsData = require('./wtchs_data'),
    childrenData = require('./children_data'),
    needsData = require('./needs_data');

/** @lends apemanfileDocBud */
function apemanfileDocBud(apemanfile) {
    if (typeof apemanfile === 'string') {
        apemanfile = require(apemanfile);
    }

    assert.ok(apemanfile, 'apemanfile is required.');

    let $cwd = apemanfile.$cwd || process.cwd(),
        $pkg = apemanfile.$pkg,
        $apps = apemanfile.$apps,
        $notes = apemanfile.$notes,
        $children = apemanfile.$children,
        $needs = apemanfile.$needs,
        $tmpls = apemanfile.$tmpls,
        $tasks = apemanfile.$tasks,
        $wtchs = apemanfile.$wtchs,
        $proto = apemanfile.$proto;

    let arrayConcat = arrayreduce.arrayConcat(),
        emptyReject = arrayfilter.emptyReject();

    return {
        force: true,
        mode: '444',
        mkdirp: true,
        tmpl: _tmpl('apdoc.hbs'),
        data: {
            pkg: pkgData($pkg),
            proto: protoData($proto),
            apps: appsData($apps, $notes),
            children: childrenData($children),
            tmpls: [
                tmplsData($tmpls) || [],
                Object.keys($children || {}).map((namespace) => {
                    let $child = $children[namespace];
                    let data = tmplsData($child.$tmpls);
                    return (data || []).map((tmpl) => {
                        tmpl.path = path.relative(
                            $cwd,
                            path.resolve($child.$cwd || process.cwd(), tmpl.path)
                        );
                        return tmpl;
                    });
                }).reduce(arrayConcat, [])
            ].reduce(arrayConcat, []).filter(emptyReject),
            tasks: [
                tasksData($tasks, $notes) || [],
                Object.keys($children || {}).map((namespace) => {
                    let $child = $children[namespace];
                    let data = tasksData($child.$tasks, $child.$notes);
                    return (data || []).map((task) => {
                        task.name = [namespace, task.name].join('#');
                        return task;
                    });
                }).reduce(arrayConcat, [])
            ].reduce(arrayConcat, []).filter(emptyReject),
            wtchs: [
                wtchsData($wtchs, $notes),
                Object.keys($children || {}).map((namespace) => {
                    let $child = $children[namespace];
                    let data = wtchsData($child.$wtchs, $child.$notes);
                    return (data || []).map((wch) => {
                        wch.name = [namespace, wch.name].join('#');
                        return wch;
                    });
                }).reduce(arrayConcat, [])
            ].reduce(arrayConcat, []).filter(emptyReject),
            needs: needsData($needs)
        }
    }
}

module.exports = apemanfileDocBud;
